# -------------------------------------------------------------------------------------
# MKDoc::XML::Mail
# -------------------------------------------------------------------------------------
#
#       Author : Jean-Michel Hiver (jhiver@mkdoc.com).
#    Copyright : (c) MKDoc Holdings Ltd, 2001
# 
#      Unauthorized modification, use, reuse, distribution or redistribution
#      of this module is stricly forbidden.
#
#    Description:
#
# -------------------------------------------------------------------------------------
package MKDoc::XML::Mail;
use strict;
use Encode qw /encode/;
use base qw /MKDoc::XML::Tree/;


sub process
{
    my $class = shift;
    my $xml_text = shift;
    local $::MKD_XML_Tree_NODES = {};
    
    my $tree = $class->new ($xml_text);
    my $headers = $class->_process_headers ($tree);
    my $body = $class->_process_body ($tree);
    return $headers . "\n\n" . $body;
}


sub _process_headers
{
    my $class = shift;
    my $tree = shift;
    my @res  = ();
    foreach my $node ($tree->children())
    {
	next if ($node->name eq '0');
	my $header = $node->name;
	if ($header !~ /^body$/i)
	{
	    my $text = join '', map { ($_->name eq '0') ? $_->{text} : () } $node->children();
	    $text =~ s/\n/ /gsm;
	    $text = encode ('MIME-Header', $text);

	    
	    push @res, "$header: $text";
	}
    }
    return join "\n", @res;
}


sub _process_body
{
    my $class = shift;
    my $tree = shift;
    unless ($tree->name eq 'body')
    {
	for ($tree->children)
	{
	    my $res = $class->_process_body ($_);
	    return $res if (defined $res);
	}
	
	return undef;
    };

    my @res = ();
    foreach my $node ($tree->children)
    {
	next if ($node->name() eq '0');
	if ($node->name() eq 'p')
	{
	    push @res, $class->_process_body_p ($node);
	}
	elsif ($node->name() eq 'pre')
	{
	    push @res, $class->_process_body_pre ($node);
	}
	else
	{
	    warn "unrecognized element: " . $node->name();
	}
    }
    
    return join "\n\n", @res;
}


sub _process_body_pre
{
    my $class = shift;
    my $node  = shift;
    my $text  = join '', map { defined $_->{is_text} ? $_->{text} : () } $node->children();
    return $text;
}


sub _process_body_p
{
    my $WRAP = 68;
    
    my $class = shift;
    my $node  = shift;
    my $text  = join '', map { defined $_->{is_text} ? $_->{text} : () } $node->children();
    $text =~ s/\n/ /gsm;
    $text =~ s/^\s+//;
    $text =~ s/\s+$//;
    my @tokens = split /\s+/, $text;
    
    my @res = ();
    my @next_line = ();
    while (@tokens)
    {
	my $next_token = shift (@tokens);
	my $potential_line = join " ", (@next_line, $next_token);
	if (length ($potential_line) > $WRAP)
	{
	    if (@next_line == 0)
	    {
		push @res, $next_token;
	    }
	    else
	    {
		unshift (@tokens, $next_token);
		push @res, join " ", @next_line;
		@next_line = ();
	    }
	}
	else
	{
	    push @next_line, $next_token;
	}
    }
    
    push @res, join " ", @next_line unless (@next_line == 0);
    return join " \n", @res;
}


1;


__END__
