# -------------------------------------------------------------------------------------
# flo::editor::File
# -------------------------------------------------------------------------------------
#
#       Author : Jean-Michel Hiver (jhiver@mkdoc.com).
#    Copyright : (c) MKDoc Holdings Ltd, 2001
# 
#      Unauthorized modification, use, reuse, distribution or redistribution
#      of this module is stricly forbidden
#
#    Description:
#
#      A class that can be used to attach any kind of file to an MKDoc document
#
# -------------------------------------------------------------------------------------
package flo::editor::File;
use MKDoc::Util::Text2HTML;
use MKDoc::Config;
use flo::Standard qw /cgi path_info raw_path_info session table template current_document/;
use flo::Editor;
use strict;
use 5.008_000;
use utf8;

use base qw /flo::Component
	     flo::editor::Mixin::compute_name
	     flo::editor::Mixin::normalize_name
	     flo::editor::Mixin::upload_file/;


sub preferred_extension
{
    my $self  = shift;
    my $file  = $self->{file} || return '';
    $file =~ s/^.*\///;
    my ($ext) = $file =~ /.*?\.(.*)/;
    $ext ||= $self->type();
    return $ext;
}


##
# $self->new;
# -----------
# Returns a new flo::editor::File object based on CGI input
##
sub new
{
    my $class = shift;
    $class    = ref $class || $class;
    my $self  = bless { @_ }, $class;
    
    my $cgi = $self->{cgi};
    my $param_name = $self->{param_name};
    
    # if CGI is defined, then we probably want to do some stuff
    if (defined $cgi)
    {
	# update title field
	if (defined $cgi->param ($param_name . "_title"))
	{
	    my $title = $cgi->param ($param_name . "_title");
	    $self->{title} = $title;
	}
	else
	{
	    $self->{title} = '';
	}
	    
	# if the upload field is not empty, then the user
	# just uploaded a new file
	if (defined $cgi->param ($param_name . "_upload"))
	{
	    $self->{title} = '';
	    my $new_file = $cgi->param ($param_name);
	    $new_file = "$new_file";
	    if ($new_file)
	    {
		my $new_file_no_slash = $new_file;
		$new_file_no_slash =~ s/^.*\///g;
		$new_file_no_slash =~ s/^.*\\//g;
		
		my $new_file_friendly = $self->normalize_name ($new_file_no_slash);
		my $dir = $self->compute_name;
		my $full_dir = MKDoc::Config->FILE_DISK_PATH . "/" . $dir;
		
		mkdir $full_dir;
		$class->upload_file ('file_disk_path', $dir, $new_file_friendly, $cgi->param ($param_name))
		    or return $self;
		
		my $self_title = $self->{title};
		$self_title ||= do {
		    my $res = $new_file_friendly;
		    $res =~ s/\..*//;
		    $res;
		};
		
		my $self_file = "$dir/$new_file_friendly";
		
		$self->{file} = $self_file;
		$self->{title} = $self_title;
	    }
	}
	else
	{
	    my $file = $cgi->param ($param_name . "_file");
	    $self->{file} = $file;
	}
    }
    
    return $self;
}


##
# $self->generate_xml;
# --------------------
# Returns the XML for that component
##
sub generate_xml
{
    my $self = shift;
    my $data = '';
    if ($self->{file})
    {
	my $src = $self->{file};
	my $title = text2html ($self->{title});
	$data = qq |<a href="$src">$title</a> />|;
    }
    
    return $data;
}


##
# $self->parse_xml;
# -----------------
# Parses the XML for that component
##
sub parse_xml
{
    my $self = shift;
    my $line = shift; # u_shift
    if ($line)
    {
	my ($file, $title) = $line =~ /\<a href=\"(.*?)\"\>(.*)\<\/a\> \/\>/;
	my $q_file_http_path = quotemeta (MKDoc::Config->FILE_HTTP_PATH);
	$file =~ s/^$q_file_http_path//;
	$file =~ s/^\///o;
	$self->{file}  = $file;
	$self->{title} = html2text ($title) || $self->{file};
    }
    else
    {
	my $empty = "";
	$self->{file} = $empty;
	$self->{title} = $empty;
    }
}


##
# $self->html;
# ------------
# Generates the (X)HTML for that component
##
sub html
{
    my $self = shift;
    return $self->generate_xml;
}


##
# $self->internal_location();
# ---------------------------
# Returns the internal location associated with $self.
##
sub internal_location
{
    my $self = shift;
    return MKDoc::Config->FILE_HTTP_PATH . "/" . $self->{file};
}


##
# $self->URI;
# -----------
# Returns the URI associated with $self.
##
sub URI
{
    my $self = shift;
    return $self->uri();
}


1;
