# -------------------------------------------------------------------------------------
# flo::plugin::Account::Details
# -------------------------------------------------------------------------------------
# Author : Steve Purkis <spurkis@mkdoc.com>
# Copyright : (c) 2003, MKDoc Holdings Ltd.
# -------------------------------------------------------------------------------------
package flo::plugin::Account::Details;
use MKDoc::XML::Mail;
use flo::Standard qw /cgi/;
use flo::HTTP::Header;
use flo::Record::Ticket;
use flo::Record::Editor;
use Carp;
use Petal;

use strict;
use 5.008_000;
use utf8;

require flo::plugin::Account::Preferences;

use base qw /flo::Plugin/;


sub template_path { '/account/details' };


sub activate
{
    my $self = shift;
    return unless ($self->SUPER::activate());
    
    my $cgi = flo::Standard::cgi();
    my $id  = $cgi ->param ('t') || return;
    $self->{ticket} = $self->validate_ticket_id ($id) || return;
    
    return 1;
}


sub http_get
{
    my $self = shift;
    $self->render_http (
	self       => $self,
	object     => $self->parent,
	__input__  => 'XML',
	__output__ => 'XHTML',
       );
    return 'TERMINATE';
}


sub http_post
{
    my $self = shift;
    $self->_validate_and_create_account();
    $self->has_errors() and return $self->http_get();
    
    $self->_send_mail();
    $self->{ticket}->deleteFromDb();
    return $self->http_get;
}


sub _send_mail
{
    my $self = shift;
    local *flo::plugin::Account::Details::template_path;
    *flo::plugin::Account::Details::template_path = sub { '/account/emails/details' };
    
    my $xml_mail = eval {
	$self->render (
	    self       => $self,
	    object     => $self->parent(),
	    __input__  => 'XML',
	    __output__ => 'XML',
	   );
    };
    defined $@ and $@ and do {
	warn $@;
	new MKDoc::Ouch 'mail/cannot_create';
	return 0;
    };
    
    eval {
	my $txt_mail = MKDoc::XML::Mail->process ($xml_mail);
	my $mail = new MKDoc::Mailer ($txt_mail);
	$mail->send();
    };
    defined $@ and $@ and do {
	warn $@;
	new MKDoc::Ouch 'mail/cannot_send';
	return 0;
    };
}


sub validate_ticket_id
{
    my $class     = shift || return;
    my $ticket_id = shift || return;
    
    # clear expired tickets:
    flo::Record::Ticket::clear_old_tickets_from_db;
    
    # try to load ticket from db
    my $ticket = new flo::Record::Ticket ($ticket_id);
    
    return $ticket;
}


sub _validate_and_create_account
{
    my $self   = shift || return;
    my $ticket = $self->{ticket};
    my $cgix   = flo::Standard::cgi();
    
    ($cgix->param ('password') eq $cgix->param ('password_verify')) or do {
	new MKDoc::Ouch 'user/password/mismatch';
	return 0;
    };
    
    $self->{user} = new flo::Record::Editor
	Login       => $cgix->param ('login'),
	Password    => $cgix->param ('password'),
	Email       => $ticket->email(),
	First_Name  => $cgix->param ('first_name'),
	Family_Name => $cgix->param ('family_name'),
	Disabled    => 0;
}


sub user
{
    my $self = shift;
    return $self->{user};
}


1;
