use strict;
use warnings;
use Cache::FileCache;
use LWP::Simple qw (head get $ua);

use constant MAX_SIZE => 5242880;	# five megabytes


our $Cache;
our $URI;
our $Resp;

our $MAX_RETRIES = shift || do { print STDERR "Usage: $0 <max_retries>"; exit };


BEGIN {
  $ENV{MKDOC_DIR} || die 'MKDOC_DIR is not defined';
  unshift @INC, $ENV{MKDOC_DIR};
};

eval "use MKDoc::Config";
die $@ if (defined $@ and $@);

eval "use MKDoc::HTTP::Response";
die $@ if (defined $@ and $@);

$ua->max_size (MAX_SIZE);
for (list_directories()) { process_directory ($_) }


sub list_directories
{
    my $file = "$ENV{MKDOC_DIR}/conf/sites.conf";

    open FP, "<$file" || die "Cannot read-open $file";
    my $data = join '', <FP>;
    close FP;

    my $VAR1 = undef;
    eval "$data";
    die $@ if (defined $@ and $@);
    
    my %RAV1 = map { $_ => 1 } values %{$VAR1};
    return sort keys %RAV1;
}


sub process_directory
{
    $ENV{SITE_DIR} = shift;
    print "$ENV{SITE_DIR}\n";
    local $Cache = cache_object();
    foreach my $key ($Cache->get_keys()) {
        local $URI  = $key;
        local $Resp = new MKDoc::HTTP::Response ($Cache->get ($URI));
        $Resp->Status() !~ /^50/ and next;
	process_uri();
	$Cache->set ($URI, $Resp->GET());
    }
}


sub process_uri
{
    my ($content_type, $document_length, $modified_time, $expires, $server) = head ($URI);
    if (defined $content_type and $content_type =~ /application\/xml/i)
    {
	print "\t200 - $URI\n";
        $Resp->clear();
	$Resp->Status ('200 OK');
        $Resp->Content_Type ($content_type);
	$Resp->Last_Modified ($modified_time) unless (defined $modified_time);
        $Resp->Body (get ($URI));
    }
    elsif ($Resp->X_Retries > $MAX_RETRIES)
    {
	print "\t404 - $URI (too many retries)\n";
        $Resp->clear();
        $Resp->Status ('404 Not Found');
        $Resp->Content_Type ('text/plain');
        $Resp->Content_Length ('0');
    }
    else
    {
        my $retries = $Resp->X_Retries();
	print "\t503 - $URI ($retries)\n";
        $Resp->Status ('503 Service Unavailable');
        $Resp->Retry_After (24 * 3600);
        $Resp->X_Retries ($Resp->X_Retries() + 1);
    }
}


sub cache_object
{
    my %opt  = (
        cache_root => MKDoc::Config->CACHE_DIR,
	namespace  => 'rss',
       );
    return new Cache::FileCache ( \%opt );
}


1;


__END__
